/* 
	AdHoc.js
	Contains:
		Methods to be used by the GUI built in the AdHocPanel class
		
		
	A panel contains the following HTML element
	
	
*/

var listOfChartIdsInRow;
var indexOfChartInRow;

var CALENDAR_URL = './../Objectstore/General/adHocCalendar.html?dt=';

// date format in URL yyyy.mm.dd.hh
var urlFormatRegexp = "\\d{4}.\\d{2}.\\d{2}.\\d{2}";
//period format in URL sdate-edate
var urlFormatPeriodRegexp = urlFormatRegexp+"-"+urlFormatRegexp;

var urlFormatRelPeriodRegexp = "R\\d{2}\[HDWMQY]";


function applyPortalPeriodTo(panelId, listOfCharts) {
    applyPeriodTo('na','na',panelId,listOfCharts);
}

function applyPeriodTo(startD,endD,panelId,listOfCharts) {
    var url = document.getElementById(panelId+'url').value;
    for ( var key in listOfCharts) 
    {
      
      url=removePreviousAHSetting(listOfCharts[key],url);
      
      if( startD!='na' && endD!='na')
          url+=buildAdhocUrlParameter(listOfCharts[key],startD,endD);
    }

    window.location = url;
}

function applyAdhocTo(panelId, listOfCharts) {
    var startDateFormatted = formatDate(panelId,'s');
    var endDateFormatted = formatDate(panelId,'e');
    
    var startDate = getTheDate(panelId,'s');
    var endDate = getTheDate(panelId,'e');
    
    if ( endDate.valueOf() < startDate.valueOf()) 
    {
       alert('Invalid absolute period : end date prior to start date');
       return;
    }
    
    applyPeriodTo(startDateFormatted,endDateFormatted,panelId,listOfCharts);    
}

function applyRelativePeriod(panelId,listOfCharts) {

         var multiplier = document.getElementById(panelId+'rel').value;
         var aregexp = /\d{1,2}/;
         if (!aregexp.test(multiplier) || multiplier<1 || multiplier > 99) 
         {
            alert('Invalid relative period : '+multiplier);
            return;
         }
         
         if(multiplier < 10 ) multiplier='0'+multiplier;
         var timespan = document.getElementById(panelId+'TS').value;         
         var url = document.getElementById(panelId+'url').value;
         for ( var key in listOfCharts) 
         {             
            url=removePreviousAHSetting(listOfCharts[key],url);            
            url+=buildRelAdhocUrlParameter(listOfCharts[key],multiplier,timespan);
         }

         window.location = url;                          
}

function formatDateObjectToURL(aDate) {

    var monthv = aDate.getMonth()+1;
    if (monthv<10) monthv='0'+monthv;
    var dayv = aDate.getDate();
    if(dayv<10) dayv='0'+dayv;
    var hourv = aDate.getHours();
    if(hourv<10) hourv='0'+hourv;
    var minutes = aDate.getMinutes();
    if(minutes<10) minutes='0'+minutes;
    
    return aDate.getFullYear()+'.'+monthv+'.'+dayv+'.'+hourv+'.'+minutes;     

}

//Update The fields when the popup calendar is closed
function calendarClosed(datetime) {
         var theDate = new Date(datetime);
         
         setTheDate(theDate,calendarToSetPanel,calendarToSetPrefix);
}

var calendarToSetPanel ='';
var calendarToSetPrefix ='';

// list of months
var MONTHS = ["January", "February", "March", "April", "May", "June",
		"July", "August", "September", "October", "November", "December"];
// list of days of the week
var WEEKDAYS = [ "M", "T", "W", "T", "F", "S","S"];

function displayCalendar(panelId, prefix, monthsTranslated, weekdaysTranslated) {
            
            MONTHS = monthsTranslated;
            WEEKDAYS = weekdaysTranslated;
            
            var theDate = getTheDate(panelId,prefix);
            
            calendarToSetPanel = panelId;
            calendarToSetPrefix = prefix;
            
            var dayselector = document.getElementById(panelId+prefix+'Day');
            
            var posX = findPosX(dayselector);
            var posY = findPosY(dayselector);
            
            var calendarWindow = window.open(CALENDAR_URL+theDate.valueOf(),'Calendar', 'width=150,height='+188+',status=no,resizable=no,top='+posY+',left='+posX+',dependent=yes,alwaysRaised=yes');
	        calendarWindow.opener = window;
	        calendarWindow.focus();	                 
}

function findPosX(obj) {
    var curleft = 0;
    if(obj.offsetParent)
        while(1) 
        {
          curleft += obj.offsetLeft;
          if(!obj.offsetParent)
            break;
          obj = obj.offsetParent;
        }
    else if(obj.x)
        curleft += obj.x;
    return curleft;
}

  function findPosY(obj) {
    var curtop = 0;
    if(obj.offsetParent)
        while(1)
        {
          curtop += obj.offsetTop;
          if(!obj.offsetParent)
            break;
          obj = obj.offsetParent;
        }
    else if(obj.y)
        curtop += obj.y;
    return curtop;
  }

function checkRB(name) {
 
 document.getElementById(name).checked = true;
}

//return 'PP' if portal period is checked in the panel
//return 'RP' if realtive period is checked in the panel
//return "AP" if ad-hoc period is checked in the panel
function getPeriodCheckedInPanel(panelId) {
         var rpradio = document.getElementById(panelId+'RPRadio');
         var apradio = document.getElementById(panelId+'APRadio');
         
         if (rpradio && rpradio.checked)
                     return 'RP';
         
         if (apradio && apradio.checked)
                     return 'AP';

         return 'PP';
}

function updateListOfCharts(panelId, listOfCharts) {
         var selectedPeriod = getPeriodCheckedInPanel(panelId);
         
         if (selectedPeriod == 'AP')
            applyAdhocTo(panelId, listOfCharts);         
         else 
         if (selectedPeriod == 'PP')
            applyPortalPeriodTo(panelId, listOfCharts);
         else
         if (selectedPeriod == 'RP')
            applyRelativePeriod(panelId,listOfCharts);
}


function singleButtonClicked(panelId, chartId) {
         updateListOfCharts(panelId, chartId);   
}

// User click on 'Row' button
function rowButtonClicked(panelId, chartId) {
         getChartsInSameRow(chartId);         
         updateListOfCharts(panelId, listOfChartIdsInRow);         
}

// User click on 'All' button
function allButtonClicked(panelId, charts) {
         var listOfCharts = new Array();
         var ind = 0;
         for(var key in charts) 
         {
                 //now test if the a chart that can accept adhoc parameter
                 var adhocSupport = document.getElementById('adhocUpdate'+charts[key]);                 
                 if (adhocSupport && adhocSupport.value.indexOf('all')>-1) 
                 {
                    listOfCharts[ind] = charts[key];
                    ind++;
                 }
         }
         updateListOfCharts(panelId, listOfCharts);         
}

// User click on 'Column' button
function columnButtonClicked(panelId, chartId) {
      
      var theTable  = findTableElement(chartId);      
      var theTableLayout = buildTableLayout(theTable);
      
    //  theTableLayout.displayLayout();
      
      //find all chart in the same column         
      var columnNumber = theTableLayout.getColumnNumber(chartId);   
       // alert('column nb='+columnNumber);          
      var column;
      if ( columnNumber != -1) 
      {
         
         column = theTableLayout.getColumn(columnNumber);
      }
      
      if (column && column.length>0) 
      {
               
        // alert('column nb='+columnNumber+' '+column.toString());
      }
      else 
      {
         column = new Array(1);
         column[0] = chartId;
         
      }
      
      updateListOfCharts(panelId, column);     
         
}



function add1Day(panelId,prefix) {

    var newDate = getTheDate(panelId,prefix);
    //Add 1 day
    newDate.setDate(newDate.getDate()+1);
    setTheDate(newDate,panelId,prefix);
            
}

function substract1Day(panelId,prefix) {
    var newDate = getTheDate(panelId,prefix);
    //Add 1 day
    newDate.setDate(newDate.getDate()-1);   
    setTheDate(newDate,panelId,prefix);            
}

function add1Hour(panelId,prefix) {
    var newDate = getTheDate(panelId,prefix);
    //Add 1 hour
    newDate.setHours(newDate.getHours()+1);    
    setTheDate(newDate,panelId,prefix);            
}

function substract1Hour(panelId,prefix) {
    var newDate = getTheDate(panelId,prefix);
    //Add 1 day
    newDate.setHours(newDate.getHours()-1);    
    setTheDate(newDate,panelId,prefix);            
}

function setTo7am(panelId,prefix) {
    var newDate = getTheDate(panelId,prefix);
    //Add 1 day
    newDate.setHours(7);    
    setTheDate(newDate,panelId,prefix);            
}

function setTo6pm(panelId,prefix) {
    var newDate = getTheDate(panelId,prefix);
    //Add 1 day
    newDate.setHours(18);    
    setTheDate(newDate,panelId,prefix);            
}

//Return the date in the panel , prefix is 's' to get start date , 'e' to get end date
function getTheDate(panelId,prefix) {
    var monthv = document.getElementById(panelId+prefix+'Month').value;
    var dayv = document.getElementById(panelId+prefix+'Day').value;
    var yearv = document.getElementById(panelId+prefix+'Year').value;
    var hourv = document.getElementById(panelId+prefix+'Hour').value;
    
    return new Date(yearv,monthv-1,dayv,hourv,0,0);
}

// set the date for the panel , prefix is 's' to set start date , 'e' to set end date
function setTheDate(newDate, panelId, prefix) {
    var monthv = newDate.getMonth()+1;
    if (monthv<10) monthv='0'+monthv;
    
    document.getElementById(panelId+prefix+'Month').value=monthv;
    
    var dayv = newDate.getDate();
    if(dayv<10) dayv='0'+dayv;
    document.getElementById(panelId+prefix+'Day').value=dayv;
    
    document.getElementById(panelId+prefix+'Year').value=newDate.getYear();
    
    var hourv = newDate.getHours();
    if(hourv<10) hourv='0'+hourv;
    document.getElementById(panelId+prefix+'Hour').value=hourv;
    
    
    checkRB(panelId+'APRadio');
}


// returns a date in URL format 
//the prefix determine what date in the given panel to return ( start date or end date) 
// prefix = 's' -> get start date
//prefix = 'e' -> get end date
function formatDate(panelId,prefix) {

    // alert( panelId+prefix+'Month');   

    var monthv = document.getElementById(panelId+prefix+'Month').value;
    var dayv = document.getElementById(panelId+prefix+'Day').value;
    var yearv = document.getElementById(panelId+prefix+'Year').value;
    var hourv = document.getElementById(panelId+prefix+'Hour').value;
    
    return yearv+'.'+monthv+'.'+dayv+'.'+hourv;
}


// build the adhoc url section for 1 chart : &ahChart1=2006.12.25.08.00-2006.12.27.18.00
function buildAdhocUrlParameter(chartId, startdate,enddate) {
         return '&ah'+chartId+'='+startdate+'-'+enddate;
}

// build the adhoc relative period url section for 1 chart : &ahChart1=2006.12.25.08.00-2006.12.27.18.00
function buildRelAdhocUrlParameter(chartId, multiplier,timespan) {
         return '&ah'+chartId+'=R'+multiplier+timespan;
}

function removePreviousAHSetting(chartId,url) {

     //Remove all absolute period from url
     var myRegExp1 = new RegExp("&ah"+chartId+"="+urlFormatPeriodRegexp);
     var cleanURl = url.replace(myRegExp1,'');
     
     //Remove all relative period from url
     var myRegExp2 = new RegExp("&ah"+chartId+"="+urlFormatRelPeriodRegexp);
     cleanURl = cleanURl.replace(myRegExp2,'');
     
     return cleanURl;    

}

/*

  Methods to find how charts are layed out

*/

function TableLayout() {
         this.rows = new Array();
         this.numberOfRows = 0;
}

TableLayout.prototype.addRow = function(aRow) {

         //Copy the row content in a new array
         this.rows[this.numberOfRows] = new Array();
         for (var i=0;i<aRow.length;i++) 
         {
             this.rows[this.numberOfRows][i] = aRow[i];
         }
         
         this.numberOfRows++;
}

//return the column number that contains the chartid
TableLayout.prototype.getColumnNumber = function(chartId) {
      
      var currentRow = 0;
      var colNumber = 0;
      
      var found = false;      
   //   alert('this.numberOfRows'+this.numberOfRows);
      while (!found && currentRow <this.numberOfRows) 
      {
            var selectedRow = this.rows[currentRow];
            
            colNumber = 0;
            while(!found && colNumber<selectedRow.length) 
            {
                 
                 
                 if (selectedRow[colNumber]==chartId) 
                 {
                    found = true;
                    
                   // alert('chart found in column '+ colNumber);
                    
                 }
                 else
                 colNumber++;         
            }
            
            currentRow++;
      }
      
      if (found) return colNumber;
      
      return -1;
            
      
}

TableLayout.prototype.getColumn = function(colNumber) {
     
     var column = new Array();
     var nbOfElements = 0;
     
     for ( var i=0;i<this.numberOfRows;i++) 
     {
       var selectedRow = this.rows[i];  
     //  alert('selected row '+i+' :' + selectedRow);
       if ( selectedRow[colNumber]) 
       {
     
          column[nbOfElements] = selectedRow[colNumber];
          nbOfElements++;
     
          
       }
     
     }                                      
                                           
     return column;                                      
}


TableLayout.prototype.getChart = function(aRowNumber , aColumnNumber) {

         if ( aRowNumber < this.numberOfRows) 
         {
            var selectedRow = this.rows[aRowNumber];
            
            if ( aColumnNumber < selectedRow.length) 
            {
               return selectedRow[aColumnNumber];
            }
            else return;
            
         }
         else return;
}         

TableLayout.prototype.displayLayout = function() {

        for ( var i=0 ; i<this.numberOfRows;i++) 
        {
        
            var arow = this.rows[i];
            
            for (var j = 0 ; j< arow.length;j++) 
            {
                alert( i +','+j+'='+arow[j]);
            }
        
        }                            

}


//This methods find the Table element that contains the chart
function findTableElement(chartId) {
         var chart = document.getElementById(chartId);
         // if the chart exist
         if (chart) 
         {
            var parent = chart.parentNode;
            //Look for the table that contains the chart
            while ( parent && parent.nodeName != 'TBODY') 
            {
                  parent = parent.parentNode;
            }
            
            return parent;
         }
         else return;                  
}

// Build a TableLayout object from the table element
function buildTableLayout(tableElement) {

         tableLayout = new TableLayout();
         
         for (var i = 0 ; i<tableElement.childNodes.length; i++ ) 
         {                                    
            if (tableElement.childNodes[i].nodeName == 'TR') 
            {
               
               //Look for the charts in that row
               listOfChartIdsInRow = new Array();
               indexOfChartInRow = 0;
               getChartsInElement(tableElement.childNodes[i],false)
              
               if(listOfChartIdsInRow && listOfChartIdsInRow.length >0 ) 
               {
                 tableLayout.addRow(listOfChartIdsInRow);
               }                                             
            }             
         }
         
         return tableLayout;         
}

// This methods find all the charts id of the charts that in the same row as the given chart
function getChartsInSameRow(chartId) {

         var chart = document.getElementById(chartId);
                  
         var parent = chart.parentNode;
                  
         //Look for the row that contains the chart
         while ( parent && parent.nodeName != 'TR') 
         {
            parent = parent.parentNode;
         }
         
         //Look for the charts in that row
         listOfChartIdsInRow = new Array();
         indexOfChartInRow = 0;
         if (parent && parent.nodeName == 'TR') 
         {
              getChartsInElement(parent,true);
         }
                  
}

function getChartsInElement(anElement, filter) {
         
         //Filter out nodes that are not element         
         if(anElement.nodeType != 1) return;        
         //---------------------------------------
         
         //The element is a leaf element
         if (!anElement.hasChildNodes()) 
         { 
           //If we found a chart ( ie the elt has an id and it is an image
           if (anElement.id && anElement.nodeName=='IMG') 
           {
              //alert(' find :'+anElement.id + ' parent :' +anElement.parentNode.nodeName+' class :'+anElement.parentNode.id);
              //now test if the iamge we have found is a chart that can accept adhoc parameter
              var adhocSupport = document.getElementById('adhocUpdate'+anElement.id);
              //alert('check if rowupdate is enable '+ adhocSupport.value.indexOf('row'));
              if (!filter || (adhocSupport && adhocSupport.value.indexOf('row')>-1)) 
              {
              
                 // find a chart id in TR :'+anElement.id;
              
                 // add the chart in the array 
                 listOfChartIdsInRow[indexOfChartInRow] = anElement.id;           
                 indexOfChartInRow++;  
              
              }
              
            
           }          
             
         }
         else 
         {              
              for (var i = 0 ; i<anElement.childNodes.length; i++ ) 
              {                                    
                  
                  getChartsInElement(anElement.childNodes[i],filter);             
              }
         }         
 }

