/*
switchBlock, v1.1, by Mike Combs
Feb 25, 04 - First version
Mar 13, 06 - Added cookie support, aligned with showTabPanel syntax

On page init, cookie is read for a list of objects (usually DIVs) to switch on or off.
Clicking link toggles classname between "sbOn" and "sbOff".
Used to work by adding "hide" as a class, but IE6 doesn't support multiple class selectors

Init page with <body onload="switchBlock();">
Params: ID to switch, cookie (1=yes, 0=no/remove), className
Sample HTML for button:
	<p><a href="#" onclick="return switchBlock('bar1');">Toggle bar1</a></p>
	<p><a href="#" onclick="return switchBlock('bar2',1,'chart');">Toggle bar2, with cookie</a></p>
	<p><a href="#" onclick="switchBlock('bar1'); return switchBlock('pie1');">Toggle bar1 and pie1</a></p>
Sample HTML for object:
	<div id="bar1" class="sbOn">This div is bar1.</div>
	<div id="bar2" class="chartOn">This div is bar2.</div>
	<div id="pie1" class="sbOn">This div is pie1.</div>
Sample CSS for object:
	div.sbOn { display: block; }
	div.chartOn { display: block; background: blue; }
	div.sbOff, div.chartOff { display: none; }

*/

function switchBlock(objectID, setCookie, rootClass) {
	var expdate = new Date();
	if (objectID != undefined)
	{
		if (rootClass == undefined) { rootClass = 'sb';	}
		setObjectClass(objectID, rootClass);

		// format of cookie is:
		// bar1>sbOn,bar2>chartOn,pie1>sbOff
		if ((setCookie == 0) || (setCookie == undefined))
		{
			// clear cookie if it exists (not implemented, they'll have to use browser)
		} else {
			// set cookie, replace if it exists
			// what class did we set it to?
			oldClasses = document.getElementById(objectID).className;
			oldClass = undefined;
			if (indexOfWord(oldClasses,rootClass + 'On') > -1) { oldClass = rootClass + 'On'; }
			if (indexOfWord(oldClasses,rootClass + 'Off') > -1) { oldClass = rootClass + 'Off';	}
			// now replace or add the state
// alert('oldCookie='+readCookie('switchBlock'));
			block = readCookie('switchBlock').split(",");
			sbCookie = '';
			weReplacedOldCookie = 0;
			for(iBlock=0; iBlock<block.length; iBlock++) {
				blockState = block[iBlock];
				if (blockState != '') {
//alert('blockState '+iBlock+' of '+block.length+'='+blockState);
					blockID = blockState.substring(0,blockState.indexOf('>'));
					blockClass = block[iBlock].substring(blockID.length+1,blockState.length);
					sbCookie = (sbCookie.length > 0) ? sbCookie + ',' : sbCookie;
					if (blockID == objectID) {
						weReplacedOldCookie = 1;
						sbCookie = sbCookie + blockID + '>' + oldClass;
					} else {
						sbCookie = sbCookie + blockID + '>' + blockClass;
					}
				}
			}
			// if we didn't find a match, add the cookie
			if (weReplacedOldCookie == 0) {
				sbCookie = (sbCookie.length > 0) ? sbCookie + ',' : sbCookie;
				sbCookie = sbCookie + objectID + ">" + oldClass;
			}
//alert('cookie='+sbCookie);
//sbCookie='';
			// set 1 hour cookie in case page is refreshed, 60000 ticks/minute
			// NOTE: This doesn't work on Opera7 from file://, due to a privacy restriction
			expdate.setTime(expdate.getTime() + (60*10000));
			c = 'switchBlock=' + sbCookie + '; expires=' + expdate.toGMTString() + '; path=/';
			document.cookie = c;
		}
	} else {
		// if no params, read and set all objects from cookie
//alert('oldCookie='+readCookie('switchBlock'));
		c=readCookie('switchBlock');
		block = readCookie('switchBlock').split(",");
		for(iBlock=0; iBlock<block.length; iBlock++) {
			blockState = block[iBlock];
			if (blockState != '') {
//alert('blockState='+blockState);
				blockID = blockState.substring(0,blockState.indexOf('>'));
				blockClass = block[iBlock].substring(blockID.length+1,blockState.length);
				setObjectClass(blockID, blockClass);
			}
		}
	}
}

function setObjectClass(objectID, theClass) {
	// send "sbOn" to set the class and state
	// send "sb" to change state

	rootClass = 'sb';
	if (theClass.substring(theClass.length-3,theClass.length) == 'Off') {
		rootClass = theClass.substring(0,theClass.length-3);
		newClass = theClass;
	} else {
		if (theClass.substring(theClass.length-2,theClass.length) == 'On') {
			rootClass = theClass.substring(0,theClass.length-2);
			newClass = theClass;
		} else {
			rootClass = theClass;
			newClass = '';
		}
	}
//alert('newClass='+rootClass);

	oldClasses = document.getElementById(objectID).className;
	oldClass = undefined;
	if (indexOfWord(oldClasses,rootClass + 'On') > -1) { oldClass = rootClass + 'On'; }
	if (indexOfWord(oldClasses,rootClass + 'Off') > -1) { oldClass = rootClass + 'Off';	}

	// set newClass to be sbOn or sbOff. If no oldClass, newClass is sbOff.
    if (newClass == '') {
		newClass = (oldClass == rootClass + 'Off') ? rootClass + 'On' : rootClass + 'Off';
	}

	// remove old class (sbOn/sbOff) from list of classes, add new class
	if (oldClasses == undefined)
	{ 
		newClasses = newClass;
	} else {
		foundLoc = indexOfWord(oldClasses,oldClass);
		newClassesL = oldClasses.substring(0,foundLoc); // empty or ends with a space
		newClassesR = oldClasses.substring(foundLoc+oldClass.length,oldClasses.length); // empty or starts with space
		newClasses = newClassesL + newClass + newClassesR;
		// alert('old classes=' + oldClasses + ' newstate=' + newState + ' new classes=' + newClasses);
	}
	document.getElementById(objectID).className = newClasses;
    // self.location = self.location;
	return true;
}

function indexOfWord(theString, theWord) {
	// find word but not if it's embedded in another word
	// return index
	if (theString == theWord) {	return 0; }
	if (theString.indexOf(' ' + theWord + ' ') > -1) { return theString.indexOf(' ' + theWord + ' ') +1; }
	if (theString.indexOf(theWord + ' ') == 0) { return 0; }
	if (theString.indexOf(' ' + theWord) == theString.length-theWord.length+1) { return theString.indexOf(' ' + theWord) +1; }
	return -1;
}

function readCookie(name) {
	var i = 0; 
	var lookfor = name + "=";
	while (i < document.cookie.length) {
		var j = i + lookfor.length;
		// alert(document.cookie + ' found ' + document.cookie.substring(i,j));
		if (document.cookie.substring(i,j) == lookfor) {
			var endstr = document.cookie.indexOf(";", j);
			if (endstr == -1) {
				endstr = document.cookie.length;
			}
			return unescape(document.cookie.substring(j, endstr));
		}
		i = document.cookie.indexOf(" ", i) + 1;
		if (i == 0) {
			break;
		}
	}
	return "";
}

